@@ -32,7 +32,6 @@
 #include "absl/strings/strip.h"
 #include "absl/types/optional.h"
 #include "absl/types/span.h"
-#include "absl/types/variant.h"
 
 namespace google {
 namespace protobuf {
@@ -113,13 +112,34 @@ Printer::Format Printer::TokenizeFormat(absl::string_view format_string,
     // a while loop, not a do/while loop.
 
     absl::string_view orig = format_string;
+    absl::string_view first_pp_directive;
     while (absl::ConsumePrefix(&format_string, "\n")) {
+      // clang-format will think a # at the beginning of the line in a raw
+      // string is a preprocessor directive and put it at the start of the line,
+      // which throws off indent calculation. Skip past those to find code that
+      // is indented more realistically.
+      if (absl::StartsWith(format_string, "#")) {
+        // We don't want to drop the first #... lines. We just want to skip
+        // through here. Remember to allow resetting later.
+        if (first_pp_directive.empty()) {
+          first_pp_directive = format_string;
+        }
+        size_t next_newline_index = format_string.find('\n');
+        if (next_newline_index != absl::string_view::npos) {
+          format_string = format_string.substr(next_newline_index);
+          continue;
+        }
+      }
       raw_string_indent = 0;
       format.is_raw_string = true;
       while (absl::ConsumePrefix(&format_string, " ")) {
         ++raw_string_indent;
       }
     }
+    // Reset if we skipped through some #... lines, so that we don't drop them.
+    if (!first_pp_directive.empty()) {
+      format_string = first_pp_directive;
+    }
 
     // If we consume the entire string, this probably wasn't a raw string and
     // was probably something like a couple of explicit newlines.
@@ -160,6 +180,9 @@ Printer::Format Printer::TokenizeFormat(absl::string_view format_string,
       if (comment_index != absl::string_view::npos) {
         line_text = line_text.substr(0, comment_index);
         if (absl::StripLeadingAsciiWhitespace(line_text).empty()) {
+          // If the first line is part of an ignored comment, consider that a
+          // first line as well.
+          is_first = false;
           continue;
         }
       }
